@@ -1,6 +1,5 @@
 // -*- mode: cpp; mode: fold -*-
 // Description								/*{{{*/
-// $Id: strutl.cc,v 1.48 2003/07/18 14:15:11 mdz Exp $
 /* ######################################################################
 
    String Util - Some useful string functions.
@@ -41,6 +40,7 @@
 #include <string.h>
 #include <time.h>
 #include <unistd.h>
+#include <wchar.h>
 
 #include <apti18n.h>
 									/*}}}*/
@@ -97,6 +97,53 @@ std::string Join(std::vector<std::string> list, const std::string &sep)
    return oss.str();
 }
 
+// Returns string display length honoring multi-byte characters
+size_t DisplayLength(StringView str)
+{
+   size_t len = 0;
+
+   const char *p = str.data();
+   const char *const end = str.end();
+
+   mbstate_t state{};
+   while (p < end)
+   {
+      wchar_t wch;
+      size_t res = mbrtowc(&wch, p, end - p, &state);
+      switch (res)
+      {
+      case 0:
+         // Null wide character (i.e. L'\0') - stop
+         p = end;
+         break;
+
+      case static_cast<size_t>(-1):
+         // Byte sequence is invalid. Assume that it's
+         // a single-byte single-width character.
+         len += 1;
+         p += 1;
+
+         // state is undefined in this case - reset it
+         state = {};
+
+         break;
+
+      case static_cast<size_t>(-2):
+         // Byte sequence is too short. Assume that it's
+         // an incomplete single-width character and stop.
+         len += 1;
+         p = end;
+         break;
+
+      default:
+         len += wcwidth(wch);
+         p += res;
+      }
+   }
+
+   return len;
+}
+
 }
 }
 									/*}}}*/
@@ -240,7 +287,8 @@ bool ParseQuoteWord(const char *&String,string &Res)
 {
    // Skip leading whitespace
    const char *C = String;
-   for (;*C != 0 && *C == ' '; C++);
+   for (; *C == ' '; C++)
+      ;
    if (*C == 0)
       return false;
    
@@ -288,7 +336,8 @@ bool ParseQuoteWord(const char *&String,string &Res)
    Res = Buffer;
    
    // Skip ending white space
-   for (;*C != 0 && isspace(*C) != 0; C++);
+   for (; isspace(*C) != 0; C++)
+      ;
    String = C;
    return true;
 }
@@ -301,7 +350,8 @@ bool ParseCWord(const char *&String,string &Res)
 {
    // Skip leading whitespace
    const char *C = String;
-   for (;*C != 0 && *C == ' '; C++);
+   for (; *C == ' '; C++)
+      ;
    if (*C == 0)
       return false;
    
@@ -692,7 +742,7 @@ int stringcasecmp(string::const_iterator A,string::const_iterator AEnd,
 }
 #endif
 									/*}}}*/
-// LookupTag - Lookup the value of a tag in a taged string		/*{{{*/
+// LookupTag - Lookup the value of a tag in a tagged string		/*{{{*/
 // ---------------------------------------------------------------------
 /* The format is like those used in package files and the method
    communication system */
@@ -773,7 +823,7 @@ std::string LookupTag(const std::string &Message, const char *TagC, const char *
 // StringToBool - Converts a string into a boolean			/*{{{*/
 // ---------------------------------------------------------------------
 /* This inspects the string to see if it is true or if it is false and
-   then returns the result. Several varients on true/false are checked. */
+   then returns the result. Several variants on true/false are checked. */
 int StringToBool(const string &Text,int Default)
 {
    char *ParseEnd;
@@ -807,10 +857,6 @@ int StringToBool(const string &Text,int Default)
 // ---------------------------------------------------------------------
 /* This converts a time_t into a string time representation that is
    year 2000 compliant and timezone neutral */
-string TimeRFC1123(time_t Date)
-{
-   return TimeRFC1123(Date, false);
-}
 string TimeRFC1123(time_t Date, bool const NumericTimezone)
 {
    struct tm Conv;
@@ -990,7 +1036,7 @@ static time_t timegm(struct tm *t)
 }
 #endif
 									/*}}}*/
-// RFC1123StrToTime - Converts a HTTP1.1 full date strings into a time_t	/*{{{*/
+// RFC1123StrToTime - Converts an HTTP1.1 full date strings into a time_t	/*{{{*/
 // ---------------------------------------------------------------------
 /* tries to parses a full date as specified in RFC7231 §7.1.1.1
    with one exception: HTTP/1.1 valid dates need to have GMT as timezone.
@@ -998,10 +1044,10 @@ static time_t timegm(struct tm *t)
    we allow them here to to be able to reuse the method. Either way, a date
    must be in UTC or parsing will fail. Previous implementations of this
    method used to ignore the timezone and assume always UTC. */
-bool RFC1123StrToTime(const char* const str,time_t &time)
+bool RFC1123StrToTime(std::string const &str,time_t &time)
 {
    unsigned short day = 0;
-   signed int year = 0; // yes, Y23K problem – we gonna worry then…
+   signed int year = 0; // yes, Y23K problem – we going to worry then…
    std::string weekday, month, datespec, timespec, zone;
    std::istringstream ss(str);
    auto const &posix = std::locale::classic();
@@ -1100,57 +1146,6 @@ bool FTPMDTMStrToTime(const char* const str,time_t &time)
    return true;
 }
 									/*}}}*/
-// StrToTime - Converts a string into a time_t				/*{{{*/
-// ---------------------------------------------------------------------
-/* This handles all 3 popular time formats including RFC 1123, RFC 1036
-   and the C library asctime format. It requires the GNU library function
-   'timegm' to convert a struct tm in UTC to a time_t. For some bizzar
-   reason the C library does not provide any such function :< This also
-   handles the weird, but unambiguous FTP time format*/
-bool StrToTime(const string &Val,time_t &Result)
-{
-   struct tm Tm;
-   char Month[10];
-
-   // Skip the day of the week
-   const char *I = strchr(Val.c_str(), ' ');
-
-   // Handle RFC 1123 time
-   Month[0] = 0;
-   if (sscanf(I," %2d %3s %4d %2d:%2d:%2d GMT",&Tm.tm_mday,Month,&Tm.tm_year,
-	      &Tm.tm_hour,&Tm.tm_min,&Tm.tm_sec) != 6)
-   {
-      // Handle RFC 1036 time
-      if (sscanf(I," %2d-%3s-%3d %2d:%2d:%2d GMT",&Tm.tm_mday,Month,
-		 &Tm.tm_year,&Tm.tm_hour,&Tm.tm_min,&Tm.tm_sec) == 6)
-	 Tm.tm_year += 1900;
-      else
-      {
-	 // asctime format
-	 if (sscanf(I," %3s %2d %2d:%2d:%2d %4d",Month,&Tm.tm_mday,
-		    &Tm.tm_hour,&Tm.tm_min,&Tm.tm_sec,&Tm.tm_year) != 6)
-	 {
-	    // 'ftp' time
-	    if (sscanf(Val.c_str(),"%4d%2d%2d%2d%2d%2d",&Tm.tm_year,&Tm.tm_mon,
-		       &Tm.tm_mday,&Tm.tm_hour,&Tm.tm_min,&Tm.tm_sec) != 6)
-	       return false;
-	    Tm.tm_mon--;
-	 }	 
-      }
-   }
-   
-   Tm.tm_isdst = 0;
-   if (Month[0] != 0)
-      Tm.tm_mon = MonthConv(Month);
-   else
-      Tm.tm_mon = 0; // we don't have a month, so pick something
-   Tm.tm_year -= 1900;
-   
-   // Convert to local time and then to GMT
-   Result = timegm(&Tm);
-   return true;
-}
-									/*}}}*/
 // StrToNum - Convert a fixed length string to a number			/*{{{*/
 // ---------------------------------------------------------------------
 /* This is used in decoding the crazy fixed length string headers in 
@@ -1258,11 +1253,6 @@ static int HexDigit(int c)
 // Hex2Num - Convert a long hex number into a buffer			/*{{{*/
 // ---------------------------------------------------------------------
 /* The length of the buffer must be exactly 1/2 the length of the string. */
-bool Hex2Num(const string &Str,unsigned char *Num,unsigned int Length)
-{
-   return Hex2Num(APT::StringView(Str), Num, Length);
-}
-
 bool Hex2Num(const APT::StringView Str,unsigned char *Num,unsigned int Length)
 {
    if (Str.length() != Length*2)
