@@ -7,20 +7,19 @@
 
 #include <config.h>
 
-#include <apt-pkg/init.h>
-#include <apt-pkg/fileutl.h>
-#include <apt-pkg/error.h>
-#include <apt-pkg/acquire-method.h>
-#include <apt-pkg/strutl.h>
-#include <apt-pkg/hashes.h>
-#include <apt-pkg/configuration.h>
 #include "aptmethod.h"
+#include <apt-pkg/configuration.h>
+#include <apt-pkg/error.h>
+#include <apt-pkg/fileutl.h>
+#include <apt-pkg/hashes.h>
+#include <apt-pkg/init.h>
+#include <apt-pkg/strutl.h>
 
-#include <stddef.h>
 #include <iostream>
-#include <string>
 #include <list>
+#include <string>
 #include <vector>
+#include <stddef.h>
 
 #include <assert.h>
 #include <errno.h>
@@ -574,8 +573,8 @@ class RredMethod : public aptMethod {
 
    protected:
       virtual bool URIAcquire(std::string const &Message, FetchItem *Itm) APT_OVERRIDE {
-	 Debug = _config->FindB("Debug::pkgAcquire::RRed", false);
-	 URI Get = Itm->Uri;
+	 Debug = DebugEnabled();
+	 URI Get(Itm->Uri);
 	 std::string Path = Get.Host + Get.Path; // rred:/path - no host
 
 	 FetchResult Res;
@@ -722,7 +721,10 @@ class RredMethod : public aptMethod {
       }
 
    public:
-      RredMethod() : aptMethod("rred", "2.0", SendConfig), Debug(false) {}
+   RredMethod() : aptMethod("rred", "2.0", SendConfig), Debug(false)
+   {
+      SeccompFlags = aptMethod::BASE | aptMethod::DIRECTORY;
+   }
 };
 
 int main(int argc, char **argv)
@@ -733,8 +735,7 @@ int main(int argc, char **argv)
    Patch patch;
 
    if (argc <= 1) {
-      RredMethod Mth;
-      return Mth.Run();
+      return RredMethod().Run();
    }
 
    // Usage: rred -t input output diff ...
