@@ -1,7 +1,7 @@
 #include <config.h>
 
-#include <apt-pkg/sourcelist.h>
 #include <apt-pkg/fileutl.h>
+#include <apt-pkg/sourcelist.h>
 
 #include <string>
 #include <stdlib.h>
@@ -32,7 +32,7 @@ TEST(SourceListTest,ParseFileDeb822)
 
    pkgSourceList sources;
    EXPECT_EQ(true, sources.Read(tempfile));
-   EXPECT_EQ(2, sources.size());
+   EXPECT_EQ(2u, sources.size());
 
    if (tempfile.empty() == false)
       unlink(tempfile.c_str());
